/*
 * CExpat
 * Copyright (c) 2001 Robert Mooney (rjmooney@aboveground.cx), All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

#ifndef _EXPATEX_H_
#define _EXPATEX_H_

#include <assert.h>
#include <fstream>
#include <iostream>
#include <map>
#include <stack>
#include <string>
#include <vector>
#include "expat.h"

using namespace std;

class CExpat
{
private:
	
	friend void _ElementStart( void *vUserData, const XML_Char *ntsElement, const XML_Char **ppszAttributes );
	friend void _ElementData( void *vUserData, const XML_Char *ntsString, int nStringLen );
	friend void _ElementEnd( void *vUserData, const XML_Char *ntsElement );

public:

	// Define the attribute map
	//
	// First field is the attribute name
	// Second field is the attribute value
	//
	typedef map< string, string > element_attribute_map_t;

	// Element data consists of an element's attributes and text value.
	//
	// E.g.  <a key="value">text<b/></a>
	//
	// Element name				: a
	// Attribute key/value pair	: key/value
	// Text						: text
	// Child element			: b
	//
	typedef struct 
	{
		element_attribute_map_t attributes;				// name=value pair
		string					content;				// text value/data enclosed in the element
		int						child_vector_index;		// index to the child vector of the child element iterator vector
	}
	element_data_t;

	// Define the element multimap
	//
	// First field is the element path
	// Second field is the element data
	//
	typedef multimap< string, element_data_t > element_map_t;

	// Define the child iterator vector
	// This is a "list" of iterators of children of an arbitrary element.
	//
	typedef vector< element_map_t::iterator > child_itor_vector_t;

	// Define the child vector vector
	// This is a "list" of vectors of iterators of children of an arbitrary element.
	// This allows easily retrieval of an arbitrary element's children.
	//
	typedef vector< child_itor_vector_t > child_vector_t;

private:

	//
	// Define the stack used when creating the element map and child vectors
	//

	// Element stack data definition
	typedef struct
	{
		element_map_t::iterator itor;
		string path;
	} 
	stack_element_t;

	// Element stack definition
	typedef stack< stack_element_t > stack_t;

	// The stack provides scope when dealing with element handlers
	stack_t m_stack;

protected:

	// Handle to the XML parser created on init
	XML_Parser m_parser_handle;

	// Expat callback functions for the start of the element, the data enclosed in the element,
	// and the end of the element, respectively.
	//
	virtual void ElementStart( void *vUserData, const XML_Char *ntsElement, const XML_Char **ppszAttributes );
	virtual void ElementData( void *vUserData, const XML_Char *ntsString, int nStringLen );
	virtual void ElementEnd( void *vUserData, const XML_Char *ntsElement );

public:

	// The child vector contains vectors of iterators that designate elements stored in
	// element_map.  It is used to iterator children of other elements stored in element_map.
	//
	// E.g.  <a> <b/> </a>
	//
	// The element map contains a data segment for element <a>.
	//
	// This data segment contains an index, child_vector_index.
	//
	// To iterate children of element <a>, reference the child_vector with child_vector_index.
	//
	// (The child vector contains vectors of iterators that designate elements stored in
	//		element_map, that are children of other elements stored in element_map.) (from above)
	//
	// Iterate the resulting vector of iterators.  The iterators designate elements stored in
	// element_map, ala, the children of element <a>.
	//
	child_vector_t m_child_vector;

	// The map below provides information regarding the elements in the document
	element_map_t m_element_map;

	// Constuctor/Deconstructor
	CExpat();
	CExpat( const char *szFilename );
	CExpat( const char *szBuffer, const int nBufferLen, const int nFinal = 0 );
	virtual ~CExpat();

	// Remove an attribute from the specified element, returns TRUE on success
	bool EraseAttribute( element_map_t::iterator parent_itor, const char *szName );

	// Returns a pointer to a child iterator vector, so that the caller can cycle through the 
	// list of children associated with the parent iterator specified.
	//
	// Returns NULL if no children exist.
	//
	child_itor_vector_t* GetChildItorVector( element_map_t::const_iterator parent_itor );

	// Insert an element as a child to parent_itor, using the specified element name, attributes and text
	// When parent_itor is not specified, the element is inserted at the root
	//
	void Insert( element_map_t::iterator parent_itor, const char *szElementName, element_attribute_map_t *amAttributes = NULL, const char *szText = NULL );
	void Insert( const char *szElementName, element_attribute_map_t *amAttributes = NULL, const char *szText = NULL );
	
	// Insert an attribute in the specified element
	void InsertAttribute( element_map_t::iterator parent_itor, const char *szName, const char *szValue );

	// Parse the specified buffer
	void Parse( const char *szBuffer, const int nBufferLen, const int nFinal = 0 );

	// Parse the specified file
	void Parse( const char *szFilename );

	// Get the content portion of an element
	const char* GetContent( element_map_t::iterator parent_itor );

	// Set the content portion of an element
	void SetContent( element_map_t::iterator parent_itor, const char *szContent );
};

// Helper type definitions
typedef CExpat::element_attribute_map_t element_attribute_map_t;
typedef CExpat::element_data_t			element_data_t;
typedef CExpat::element_map_t			element_map_t;
typedef CExpat::child_itor_vector_t		child_itor_vector_t;
typedef CExpat::child_vector_t			child_vector_t;

#endif // _EXPATEX_H_
